#!/bin/zsh
set -eo pipefail
let jobs=$(nproc)-2

ARCH=$(uname -m)
CROSS_COMPILE=
CC=
OPTS=(--clang)

zparseopts -D -K -clang+=OPTS -gcc+=OPTS -arm64+=OPTS

case ${OPTS[-1]} in
--clang) CC=clang ;;
--gcc) CC=gcc ;;
--arm64) ARCH=arm64
	 CC=gcc
	 CROSS_COMPILE=aarch64-linux-gnu-
	 ;;
esac

if [ -f .git/config  ] && [ -f .git/HEAD ]; then
	REV=$(git rev-parse --short HEAD)$(git diff --quiet || echo "-dirty")
	scripts/config --set-str CONFIG_LOCALVERSION "-g$REV"
fi

make -j$jobs CC=$CC ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE $* | python -u /home/anna/bin/colors/make.py

if [[ "$CC" == "clang" ]]; then
	scripts/clang-tools/gen_compile_commands.py &
fi
